@@ -21,7 +21,7 @@
   does not depend on any other GNU software.
 */
 
-/*$Id: gdevx.c,v 1.12 1999/09/14 22:45:05 masata-y Exp $ */
+/*$Id: gdevx.c,v 1.14 2000/05/01 18:07:47 masata-y Exp $ */
 /* X Windows driver for Ghostscript library */
 /* The X include files include <sys/types.h>, which, on some machines */
 /* at least, define uint, ushort, and ulong, which std.h also defines. */
@@ -168,23 +168,21 @@ const gx_device_X gs_x11_device =
     NULL, NULL,			/* dpy, scr */
 				/* (connection not initialized) */
     NULL,			/* vinfo */
-#ifndef DISPLAYGHOSTSCRIPT
     (Colormap) None,		/* cmap */
     (Window) None,		/* win */
     NULL,			/* gc */
-    (Pixmap) 0,			/* bpixmap */
-    (Pixmap) 0,			/* dest */
-    { identity_matrix_body },	/* initial matrix (filled in) */
-#endif
     (Window) None,		/* pwin */
+    (Pixmap) 0,			/* bpixmap */
     gs_x_gs,			/* owner */
     (Window) None,		/* mwin */
 #if HaveStdCMap
     NULL,			/* std_cmap */
 #endif
+    {identity_matrix_body},	/* initial matrix (filled in) */
     (Atom) 0, (Atom) 0, (Atom) 0,	/* Atoms: NEXT, PAGE, DONE */
-    0L, (ulong) ~ 0L,		/* colors_or, colors_and */
     {0, 0, 0, 0}, 0, 0,		/* update, up_area, up_count */
+    (Pixmap) 0,			/* dest */
+    0L, (ulong) ~ 0L,		/* colors_or, colors_and */
     {				/* cp */
 	(Pixmap) 0,		/* pixmap */
 	NULL,			/* gc */
@@ -305,6 +303,7 @@ x_close(gx_device * dev)
 	xdev->color_to_rgb_size = 0;
     }
     
+/* Why is this test needed? Are these fonts shared with all devices?? */
     if (xdev->owner != gs_x_dpsagent)
       {
 	while (xdev->regular_fonts) {
@@ -562,7 +561,7 @@ x_map_rgb_color(register gx_device * dev,
 	xcp->next = (*xdev->dynamic_colors)[i];
 	(*xdev->dynamic_colors)[i] = xcp;
 	xdev->dynamic_allocs++;
-	if (XAllocColor(xdev->dpy, xdev_cmap_inline(xdev), &xc)) {
+	if (XAllocColor(xdev->dpy, xdev->cmap, &xc)) {
 	    xcp->color.pixel = xc.pixel;
 	    xcp->color.pad = True;
 	    if_debug4('C', "[cX]%u,%u,%u (dynamic) => added %lu\n",
@@ -732,19 +731,17 @@ private void
 x_get_initial_matrix(register gx_device * dev, register gs_matrix * pmat)
 {
     gx_device_X *xdev = (gx_device_X *) dev;
-    gs_matrix *initial_matrix;
 
     if ( xdev->owner == gs_x_gs ) {
 	gx_default_get_initial_matrix(dev, pmat);
 	return;
     }
-    initial_matrix = xdev_initial_matrix_inline(xdev);
-    pmat->xx = initial_matrix->xx;
-    pmat->xy = initial_matrix->xy;
-    pmat->yx = initial_matrix->yx;
-    pmat->yy = initial_matrix->yy;
-    pmat->tx = initial_matrix->tx;
-    pmat->ty = initial_matrix->ty;
+    pmat->xx = xdev->initial_matrix.xx;
+    pmat->xy = xdev->initial_matrix.xy;
+    pmat->yx = xdev->initial_matrix.yx;
+    pmat->yy = xdev->initial_matrix.yy;
+    pmat->tx = xdev->initial_matrix.tx;
+    pmat->ty = xdev->initial_matrix.ty;
 #if 0
     pmat->xx = 1;
     pmat->xy = 0;
@@ -775,12 +772,12 @@ x_send_event(gx_device * dev, Atom msg)
 
     event.xclient.type = ClientMessage;
     event.xclient.display = xdev->dpy;
-    event.xclient.window = xdev_win_inline(xdev);
+    event.xclient.window = xdev->win;
     event.xclient.message_type = msg;
     event.xclient.format = 32;
     event.xclient.data.l[0] = xdev->mwin;
-    event.xclient.data.l[1] = xdev_dest_inline(xdev);
-    XSendEvent(xdev->dpy, xdev_win_inline(xdev), False, 0, &event);
+    event.xclient.data.l[1] = xdev->dest;
+    XSendEvent(xdev->dpy, xdev->win, False, 0, &event);
 }
 
 /* Output "page" */
@@ -818,7 +815,7 @@ x_fill_rectangle(register gx_device * dev,
     set_fill_style(FillSolid);
     set_fore_color(color);
     set_function(GXcopy);
-    XFillRectangle(xdev->dpy, xdev_dest_inline(xdev), xdev_gc_inline(xdev), x, y, w, h);
+    XFillRectangle(xdev->dpy, xdev->dest, xdev->gc, x, y, w, h);
     /* If we are filling the entire screen, reset */
     /* colors_or and colors_and.  It's wasteful to do this */
     /* on every operation, but there's no separate driver routine */
@@ -835,7 +832,7 @@ x_fill_rectangle(register gx_device * dev,
 		    while (xcp) {
 			next = xcp->next;
 			if (xcp->color.pad) {
-			    XFreeColors(xdev->dpy, xdev_cmap_inline(xdev),
+			    XFreeColors(xdev->dpy, xdev->cmap,
 					&xcp->color.pixel, 1, 0);
 			}
 			gs_free((char *)xcp, sizeof(x11color), 1,
@@ -851,7 +848,7 @@ x_fill_rectangle(register gx_device * dev,
 	}
 	xdev->colors_or = xdev->colors_and = color;
     }
-    if (xdev_bpixmap_inline(xdev) != (Pixmap) 0) {
+    if (xdev->bpixmap != (Pixmap) 0) {
 	x_update_add(dev, x, y, w, h);
     }
     if_debug5('F', "[F] fill (%d,%d):(%d,%d) %ld\n",
@@ -933,16 +930,16 @@ x_copy_mono(register gx_device * dev,
     xdev->image.format = XYBitmap;
     set_function(function);
     if (bc != xdev->back_color) {
-	XSetBackground(xdev->dpy, xdev_gc_inline(xdev), (xdev->back_color = bc));
+	XSetBackground(xdev->dpy, xdev->gc, (xdev->back_color = bc));
     }
     if (fc != xdev->fore_color) {
-	XSetForeground(xdev->dpy, xdev_gc_inline(xdev), (xdev->fore_color = fc));
+	XSetForeground(xdev->dpy, xdev->gc, (xdev->fore_color = fc));
     }
     if (zero != gx_no_color_index)
 	note_color(zero);
     if (one != gx_no_color_index)
 	note_color(one);
-    put_image(xdev->dpy, xdev_dest_inline(xdev), xdev_gc_inline(xdev), &xdev->image,
+    put_image(xdev->dpy, xdev->dest, xdev->gc, &xdev->image,
 	      sourcex, 0, x, y, w, h);
 
     goto out;
@@ -955,7 +952,7 @@ x_copy_mono(register gx_device * dev,
 
 	/* Create the clipping pixmap, depth must be 1. */
 	xdev->cp.pixmap =
-	    XCreatePixmap(xdev->dpy, xdev_win_inline(xdev), raster << 3, h, 1);
+	    XCreatePixmap(xdev->dpy, xdev->win, raster << 3, h, 1);
 	if (xdev->cp.pixmap == (Pixmap) 0) {
 	    lprintf("x_copy_mono: can't allocate pixmap\n");
 	    return_error(gs_error_VMerror);
@@ -986,21 +983,21 @@ x_copy_mono(register gx_device * dev,
 	      &xdev->image, sourcex, 0, 0, 0, w, h);
 
     /* Install as clipmask. */
-    XSetClipMask(xdev->dpy, xdev_gc_inline(xdev), xdev->cp.pixmap);
-    XSetClipOrigin(xdev->dpy, xdev_gc_inline(xdev), x, y);
+    XSetClipMask(xdev->dpy, xdev->gc, xdev->cp.pixmap);
+    XSetClipOrigin(xdev->dpy, xdev->gc, x, y);
 
     /*
      * Draw a solid rectangle through the raster clip mask.
      * Note fill style is guaranteed to be solid from above.
      */
-    XFillRectangle(xdev->dpy, xdev_dest_inline(xdev), xdev_gc_inline(xdev), x, y, w, h);
+    XFillRectangle(xdev->dpy, xdev->dest, xdev->gc, x, y, w, h);
 
     /* Tidy up.  Free the pixmap if it's big. */
-    XSetClipMask(xdev->dpy, xdev_gc_inline(xdev), None);
+    XSetClipMask(xdev->dpy, xdev->gc, None);
     if (raster * h > X_MAX_TEMP_PIXMAP)
 	free_cp(dev);
 
-  out:if (xdev_bpixmap_inline(xdev) != (Pixmap) 0) {
+  out:if (xdev->bpixmap != (Pixmap) 0) {
 	/* We wrote to the pixmap, so update the display now. */
 	x_update_add(dev, x, y, w, h);
     }
@@ -1054,7 +1051,7 @@ x_copy_color(register gx_device * dev,
 		pixel = (pixel << 8) + *ptr++;
 	}
 	set_fore_color(pixel);
-	XDrawPoint(xdev->dpy, xdev_dest_inline(xdev), xdev_gc_inline(xdev), x, y);
+	XDrawPoint(xdev->dpy, xdev->dest, xdev->gc, x, y);
     } else {
 	xdev->image.width = sourcex + w;
 	xdev->image.height = h;
@@ -1065,11 +1062,11 @@ x_copy_color(register gx_device * dev,
 	xdev->image.bits_per_pixel = depth;
 	if (XInitImage(&xdev->image) == 0)
 	    return_error(gs_error_unknownerror);
-	XPutImage(xdev->dpy, xdev_dest_inline(xdev), xdev_gc_inline(xdev), &xdev->image,
+	XPutImage(xdev->dpy, xdev->dest, xdev->gc, &xdev->image,
 		  sourcex, 0, x, y, w, h);
 	xdev->image.depth = xdev->image.bits_per_pixel = 1;
     }
-    if (xdev_bpixmap_inline(xdev) != (Pixmap) 0)
+    if (xdev->bpixmap != (Pixmap) 0)
 	x_update_add(dev, x, y, w, h);
     if_debug4('F', "[F] copy_color (%d,%d):(%d,%d)\n",
 	      x, y, w, h);
@@ -1157,37 +1154,36 @@ x_put_params(gx_device * dev, gs_param_list * plist)
 	int dh = dev->height - height;
 	double qx = dev->HWResolution[0] / xres;
 	double qy = dev->HWResolution[1] / xres;
-	gs_matrix *initial_matrix = xdev_initial_matrix_inline(xdev);
 
 	if (dw != 0 || dh != 0) {
-	    XResizeWindow(xdev->dpy, xdev_win_inline(xdev),
+	    XResizeWindow(xdev->dpy, xdev->win,
 			  dev->width, dev->height);
-	    if (xdev_bpixmap_inline(xdev) != (Pixmap) 0) {
-		XFreePixmap(xdev->dpy, xdev_bpixmap_inline(xdev));
-		xdev_set_bpixmap_inline(xdev, (Pixmap) 0);
+	    if (xdev->bpixmap != (Pixmap) 0) {
+		XFreePixmap(xdev->dpy, xdev->bpixmap);
+		xdev->bpixmap = (Pixmap) 0;
 	    }
-	    xdev_set_dest_inline(xdev, (Pixmap) 0);
+	    xdev->dest = 0;
 	    gdev_x_clear_window(xdev);
 	}
 	/* Attempt to update the initial matrix in a sensible way. */
 	/* The whole handling of the initial matrix is a hack! */
-	if (initial_matrix->xy == 0) {
-	    if (initial_matrix->xx < 0) {	/* 180 degree rotation */
-		initial_matrix->tx += dw;
+	if (xdev->initial_matrix.xy == 0) {
+	    if (xdev->initial_matrix.xx < 0) {	/* 180 degree rotation */
+		xdev->initial_matrix.tx += dw;
 	    } else {		/* no rotation */
-		initial_matrix->ty += dh;
+		xdev->initial_matrix.ty += dh;
 	    }
 	} else {
-	    if (initial_matrix->xy < 0) {	/* 90 degree rotation */
-		initial_matrix->tx += dh;
-		initial_matrix->ty += dw;
+	    if (xdev->initial_matrix.xy < 0) {	/* 90 degree rotation */
+		xdev->initial_matrix.tx += dh;
+		xdev->initial_matrix.ty += dw;
 	    } else {		/* 270 degree rotation */
 	    }
 	}
-	initial_matrix->xx *= qx;
-	initial_matrix->xy *= qx;
-	initial_matrix->yx *= qy;
-	initial_matrix->yy *= qy;
+	xdev->initial_matrix.xx *= qx;
+	xdev->initial_matrix.xy *= qx;
+	xdev->initial_matrix.yx *= qy;
+	xdev->initial_matrix.yy *= qy;
     }
     return 0;
 }
@@ -1243,10 +1239,10 @@ x_strip_tile_rectangle(register gx_device * dev, const gx_strip_bitmap * tiles,
 		x_pixel pixel = (ptr[tx >> 3] & mask ? one : zero);
 
 		set_fore_color(pixel);
-		XDrawPoint(xdev->dpy, xdev_dest_inline(xdev), xdev_gc_inline(xdev), i, j);
+		XDrawPoint(xdev->dpy, xdev->dest, xdev->gc, i, j);
 	    }
 	}
-	if (xdev_bpixmap_inline(xdev) != (Pixmap) 0) {
+	if (xdev->bpixmap != (Pixmap) 0) {
 	    x_update_add(dev, x, y, w, h);
 	}
 	return 0;
@@ -1271,8 +1267,8 @@ x_strip_tile_rectangle(register gx_device * dev, const gx_strip_bitmap * tiles,
     /* Use the tile to fill the rectangle */
     set_fill_style(FillTiled);
     set_function(GXcopy);
-    XFillRectangle(xdev->dpy, xdev_dest_inline(xdev), xdev_gc_inline(xdev), x, y, w, h);
-    if (xdev_bpixmap_inline(xdev) != (Pixmap) 0) {
+    XFillRectangle(xdev->dpy, xdev->dest, xdev->gc, x, y, w, h);
+    if (xdev->bpixmap != (Pixmap) 0) {
 	x_update_add(dev, x, y, w, h);
     }
     if_debug6('F', "[F] tile (%d,%d):(%d,%d) %ld,%ld\n",
@@ -1350,7 +1346,7 @@ x_begin_typed_image(gx_device * dev,
 	srcy = (int)(src.p.y + 0.5);
 	destx = (int)(dest.p.x + 0.5);
 	desty = (int)(dest.p.y + 0.5);
-	XCopyArea(xdev->dpy, xdev_bpixmap_inline(xdev), xdev_bpixmap_inline(xdev), xdev_gc_inline(xdev),
+	XCopyArea(xdev->dpy, xdev->bpixmap, xdev->bpixmap, xdev->gc,
 		  srcx, srcy, size.x, size.y, destx, desty);
 	x_update_add(dev, destx, desty, size.x, size.y);
     }
@@ -1412,9 +1408,9 @@ x_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
 	 ******/
 #if 0				/*************** */
     if (unread) {
-	XSetGraphicsExposures(xdev->dpy, xdev_gc_inline(xdev), True);
-	XGetWindowAttributes(xdev->dpy, xdev_win_inline(xdev), &attributes);
-	XSelectInput(xdev->dpy, xdev_win_inline(xdev),
+	XSetGraphicsExposures(xdev->dpy, xdev->gc, True);
+	XGetWindowAttributes(xdev->dpy, xdev->win, &attributes);
+	XSelectInput(xdev->dpy, xdev->win,
 		     attributes.your_event_mask | ExposureMask);
     }
 #endif /*************** */
@@ -1427,7 +1423,7 @@ x_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
 	band = 1;
     for (y = y0; y < y1; y += h) {
 	h = min(band, y1 - y);
-	image = XGetImage(xdev->dpy, xdev_dest_inline(xdev), x0, y, x1 - x0, h,
+	image = XGetImage(xdev->dpy, xdev->dest, x0, y, x1 - x0, h,
 			  plane_mask, ZPixmap);
 	{
 	    int cy;
@@ -1445,7 +1441,7 @@ x_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
 	*unread = 0;
 #if 0				/*************** */
 	/* Read any exposure events. */
-	XWindowEvent(xdev->dpy, xdev_win_inline(xdev), ExposureMask, &event);
+	XWindowEvent(xdev->dpy, xdev->win, ExposureMask, &event);
 	if (event.type == GraphicsExpose) {
 	    gs_int_rect *rects = (gs_int_rect *)
 	    gs_alloc_bytes(dev->memory, sizeof(gs_int_rect),
@@ -1476,8 +1472,8 @@ x_get_bits_rectangle(gx_device * dev, const gs_int_rect * prect,
 	    }
 	}
 	/* Restore the window state. */
-	XSetGraphicsExposures(xdev->dpy, xdev_gc_inline(xdev), False);
-	XSelectInput(xdev->dpy, xdev_win_inline(xdev), attributes.your_event_mask);
+	XSetGraphicsExposures(xdev->dpy, xdev->gc, False);
+	XSelectInput(xdev->dpy, xdev->win, attributes.your_event_mask);
 #endif /*************** */
     }
     return code;
@@ -1502,7 +1498,7 @@ set_tile(register gx_device * dev, register const gx_strip_bitmap * tile)
 	xdev->ht.pixmap == (Pixmap) 0) {
 	if (xdev->ht.pixmap != (Pixmap) 0)
 	    XFreePixmap(xdev->dpy, xdev->ht.pixmap);
-	xdev->ht.pixmap = XCreatePixmap(xdev->dpy, xdev_win_inline(xdev),
+	xdev->ht.pixmap = XCreatePixmap(xdev->dpy, xdev->win,
 					tile->size.x, tile->size.y,
 					xdev->vinfo->depth);
 	if (xdev->ht.pixmap == (Pixmap) 0)
@@ -1531,11 +1527,11 @@ set_tile(register gx_device * dev, register const gx_strip_bitmap * tile)
 	dputc('\n');
     }
 #endif
-    XSetTile(xdev->dpy, xdev_gc_inline(xdev), xdev->ht.no_pixmap);	/* *** X bug *** */
+    XSetTile(xdev->dpy, xdev->gc, xdev->ht.no_pixmap);	/* *** X bug *** */
     set_function(GXcopy);
-    put_image(xdev->dpy, xdev->ht.pixmap, xdev_gc_inline(xdev), &xdev->image,
+    put_image(xdev->dpy, xdev->ht.pixmap, xdev->gc, &xdev->image,
 	      0, 0, 0, 0, tile->size.x, tile->size.y);
-    XSetTile(xdev->dpy, xdev_gc_inline(xdev), xdev->ht.pixmap);
+    XSetTile(xdev->dpy, xdev->gc, xdev->ht.pixmap);
     xdev->ht.id = tile->id;
     return xdev->useXSetTile;
 }
@@ -1551,31 +1547,31 @@ gdev_x_set_drawable(register gx_device_X *xdev, Drawable draw)
     Window root;
     int x, y, width, height, bw, depth;
 
-    xdev_set_win_inline(xdev, (Window)draw);
+    xdev->win = (Window)draw;
     
-    if (!xdev_win_inline(xdev))
+    if (!xdev->win)
 	return 0;
 
     /* read the size of the new Drawable in */
-    status = XGetGeometry(xdev->dpy, (Drawable)xdev_win_inline(xdev), &root, &x, &y, &width, &height, &bw, &depth);
+    status = XGetGeometry(xdev->dpy, (Drawable)xdev->win, &root, &x, &y, &width, &height, &bw, &depth);
  
     /* error check */
     if (!status)
 	return -1;
 
-    xdev_set_dest_inline(xdev, (Pixmap)xdev_win_inline(xdev));
+    xdev->dest = (Pixmap)xdev->win;
     if (0) dprintf3("set X drawable to %X, width %d, height %d\n", 
-		    (unsigned)xdev_win_inline(xdev), width, height);
+		    (unsigned)xdev->win, width, height);
  
     /* actually set things up */
     xdev->width = width;
     xdev->height = height;
-#ifdef DISPLAYGHOSTSCRIPT
-    {
-        gx_device_X_state *state = xdev_state_inline(xdev);
-        state->width = width;
-        state->height = height;
-    }
+#if DISPLAYGHOSTSCRIPT
+    /* We should copy more codes from gdevini.c:gdev_x_open (). - Jet */
+    xdev->MediaSize[0] =
+      (float)xdev->width / xdev->x_pixels_per_inch * 72;
+    xdev->MediaSize[1] =
+      (float)xdev->height / xdev->y_pixels_per_inch * 72;
 #endif
 #if 0
      /* offset margin values */
@@ -1601,7 +1597,7 @@ update_do_flush(register gx_device * dev)
 	int xo = xdev->update.xo, yo = xdev->update.yo;
 
 	set_function(GXcopy);
-	XCopyArea(xdev->dpy, xdev_bpixmap_inline(xdev), xdev_win_inline(xdev), xdev_gc_inline(xdev),
+	XCopyArea(xdev->dpy, xdev->bpixmap, xdev->win, xdev->gc,
 		  xo, yo, xdev->update.xe - xo, xdev->update.ye - yo,
 		  xo, yo);
 	update_init(dev);
@@ -1609,7 +1605,7 @@ update_do_flush(register gx_device * dev)
 }
 
 /* Add a region to be updated. */
-/* This is only called if xdev_bpixmap_inline(xdev) != 0. */
+/* This is only called if xdev->bpixmap != 0. */
 void
 x_update_add(gx_device * dev, int xo, int yo, int w, int h)
 {
@@ -1750,155 +1746,3 @@ alt_put_image(gx_device * dev, Display * dpy, Drawable win, GC gc,
 	XSetForeground(dpy, gc, gcv.foreground);
     return 0;
 }
-
-#if defined(DISPLAYGHOSTSCRIPT)
-/* why can't they make these header files include what they need?! */
-#include "ghost.h"
-#include "gxalloc.h"
-#include "igstate.h"
-   
-/* code to deal with driver data on a per-gstate basis */
-private void *xdev_gstate_alloc(P1(gs_memory_t *mem));
-private int xdev_gstate_copy(P2(void *to, const void *from));
-private void xdev_gstate_free(P2(void *old, gs_memory_t *mem));
-private void xdev_gstate_willchange(P2(gs_state *state, void *olddata));
-private void xdev_gstate_didchange(P2(gs_state *state, void *newdata));
-const gs_state_driverdata_procs xdev_gstate_procs = {
-	xdev_gstate_alloc,
-	xdev_gstate_copy,
-	xdev_gstate_free,
-	NULL,
-        xdev_gstate_willchange,
-        xdev_gstate_didchange
-};
-
-/* Allocate the device's part of a graphics state. */
-private void *
-xdev_gstate_alloc(gs_memory_t *mem)
-{       /* no refs, so we can do this... */
-        /* This is old code. 
-	   The memory allocated by gs_alloc_bytes is tracked by gs's GC.
-	   void *p = gs_alloc_bytes(mem, sizeof(gx_device_X_state), "xdev_gstate"); */
-        /* This is new code.
-           The memory allocated by gs_malloc is not tracked by gs's GC.
-           We have responsibilities to free the memory */
-        void *p = gs_malloc(sizeof(gx_device_X_state), 1, "xdev_gstate_alloc");
-	if (p) {
-	  gx_device_X_state *pState = (gx_device_X_state*)p;
-	  pState->initial_matrix.xx = 1.0;
-	  pState->initial_matrix.xy = 0.0;
-	  pState->initial_matrix.yx = 0.0;
-	  pState->initial_matrix.yy = 1.0;
-	  pState->initial_matrix.tx = 0.0;
-	  pState->initial_matrix.ty = 0.0;
-	} else {
-	  eprintf("gs: cannot allocate state for X device.\n");
-	  /* return_error(gs_error_VMerror); */
-	  return NULL;
-	}
-        return p;
-}
-
-/* Copy the device's part of a graphics state. */
-private int
-xdev_gstate_copy(void *to, const void *from)
-{
-#if 0
-  gx_device_X_state *xto = (gx_device_X_state *)to;
-  gx_device_X_state *xfrom = (gx_device_X_state *)from;
-  dprintf2("gstate copy from %08X to %08X\n", (unsigned)from, (unsigned)to);
-  dprintf2("copying yy from %g over %g\n", (float)xfrom->initial_matrix.yy, (float)xto->initial_matrix.yy);
-  dprintf2("copying ty from %g over %g\n", (float)xfrom->initial_matrix.ty, (float)xto->initial_matrix.ty);
-#endif
-        *(gx_device_X_state *)to = *(gx_device_X_state *)from;
-	return 0;
-}
-
-/* Free the device's part of a graphics state. */
-private void
-xdev_gstate_free(void *old, gs_memory_t *mem)
-{	
-  /* This is old code.
-     The memory allocated by gs_alloc_bytes must be 
-     freed by gs_free_object
-  gs_free_object(mem, old, "xdev_gstate"); */
-  /* This is new code.
-     The memory allocated by gs_malloc must be
-     freed by gs_free? */
-  gs_free(old, 1, sizeof(gx_device_X_state), "xdev_gstate_free");
-}
-
-/* When the gstate changes, we need to flush things out. */
-private void
-xdev_gstate_willchange(gs_state *oldstate, void *olddata)
-{       /*gx_device_X_state *data = (gs_device_X_state *)olddata;*/
-        gx_device_X *xdev = (gx_device_X *)gs_currentdevice(oldstate);
-        if (xdev->dname != gs_x11_type_name)
-          return;
-        update_flush((gx_device *)xdev);
-        /* ! we need to flush out fonts, dithering and a buncha stuff here */
-}
-
-/* When the gstate changes, we need to set things up. */
-private void
-xdev_gstate_didchange(gs_state *oldstate, void *newdata)
-{       gx_device_X_state *data = (gx_device_X_state *)newdata;
-        gx_device_X *xdev = (gx_device_X *)gs_currentdevice(oldstate);
-        if (xdev->dname != gs_x11_type_name)
-          return;
-        xdev->width = data->width;
-        xdev->height = data->height;
-}
-
-#if defined(DEBUG)
-/* non-inlines for DISPLAYGHOSTSCRIPT users ... here to enforce code cleanliness */
-gx_device_X_state * xdev_state_inline(gx_device_X const *xdev)
-{        return (gx_device_X_state*)(gs_state_driver_data_inline(igs));
-}
-
-Colormap xdev_cmap_inline(gx_device_X const *xdev)
-{        return ((gx_device_X_state*)(gs_state_driver_data_inline(igs)))->cmap;
-}
-
-void xdev_set_cmap_inline(gx_device_X *xdev, Colormap _cmap)
-{        ((gx_device_X_state*)(gs_state_driver_data_inline(igs)))->cmap = _cmap;
-}
-
-Window xdev_win_inline(gx_device_X const *xdev)
-{        return ((gx_device_X_state*)(gs_state_driver_data_inline(igs)))->win;
-}
-
-void xdev_set_win_inline(gx_device_X *xdev, Window _win)
-{        ((gx_device_X_state*)(gs_state_driver_data_inline(igs)))->win = _win;
-}
-
-GC xdev_gc_inline(gx_device_X const *xdev)
-{        return ((gx_device_X_state*)(gs_state_driver_data_inline(igs)))->gc;
-}
-
-void xdev_set_gc_inline(gx_device_X *xdev, GC _gc)
-{        ((gx_device_X_state*)(gs_state_driver_data_inline(igs)))->gc = _gc;
-}
-
-Pixmap xdev_bpixmap_inline(gx_device_X *xdev)
-{        return ((gx_device_X_state*)(gs_state_driver_data_inline(igs)))->bpixmap;
-}
-
-void xdev_set_bpixmap_inline(gx_device_X *xdev, Pixmap _bpixmap)
-{        ((gx_device_X_state*)(gs_state_driver_data_inline(igs)))->bpixmap = _bpixmap;
-}
-
-Pixmap xdev_dest_inline(gx_device_X *xdev)
-{        return ((gx_device_X_state*)(gs_state_driver_data_inline(igs)))->dest;
-}
-
-void xdev_set_dest_inline(gx_device_X *xdev, Pixmap _dest)
-{        ((gx_device_X_state*)(gs_state_driver_data_inline(igs)))->dest = _dest;
-}
-
-gs_matrix *xdev_initial_matrix_inline(gx_device_X *xdev)
-{        return &(((gx_device_X_state*)(gs_state_driver_data_inline(igs)))->initial_matrix);
-}
-
-#endif
-#endif
