@@ -21,7 +21,7 @@
   does not depend on any other GNU software.
 */
 
-/*$Id: gdevxini.c,v 1.5 1999/04/01 05:31:20 fedor Exp $ */
+/*$Id: gdevxini.c,v 1.6 2000/04/04 17:02:32 cigas Exp $ */
 /* X Windows driver initialization for Ghostscript library */
 #include "math_.h"
 #include "memory_.h"
@@ -30,10 +30,6 @@
 #include "gserrors.h"
 #include "gxdevice.h"
 #include "gdevx.h"
-#ifdef DISPLAYGHOSTSCRIPT
-#include "gxstate.h"
-extern gs_state *igs; /*! YOUCH */
-#endif
 
 extern char *getenv(P1(const char *));
 
@@ -155,10 +151,6 @@ typedef enum {
 /* Forward references */
 private int gdev_x_setup_colors(P1(gx_device_X *));
 private void gdev_x_setup_fontmap(P1(gx_device_X *));
-#ifdef DISPLAYGHOSTSCRIPT
-/* defined in gdevx.c */
-extern const gs_state_driverdata_procs xdev_gstate_procs;
-#endif
 
 /* Catch the alloc error when there is not enough resources for the
  * backing pixmap.  Automatically shut off backing pixmap and let the
@@ -202,9 +194,6 @@ gdev_x_open(register gx_device_X * xdev)
     int zero = 0;
     int xid_height, xid_width;
     int code;
-#ifdef DISPLAYGHOSTSCRIPT
-    gx_device_X_state *state;
-#endif
 
 #ifdef DEBUG
 # ifdef have_Xdebug
@@ -222,22 +211,20 @@ gdev_x_open(register gx_device_X * xdev)
 		 (dispname == NULL ? "(null)" : dispname));
 	return_error(gs_error_ioerror);
     }
-#ifdef DISPLAYGHOSTSCRIPT
-    if (!(state = (gx_device_X_state*)gs_malloc(sizeof(gx_device_X_state), 1, "gdev_x_open"))) {
-        eprintf("gs: cannot allocate state for X device.\n");
-        return_error(gs_error_VMerror);
-    }
-    /*! igs again.. ick */
-    memset(state, 0, sizeof(*state));
-    state->initial_matrix.xx = 1.0;
-    state->initial_matrix.xy = 0.0;
-    state->initial_matrix.yx = 0.0;
-    state->initial_matrix.yy = 1.0;
-    state->initial_matrix.tx = 0.0;
-    state->initial_matrix.ty = 0.0;
-    gs_state_set_driver(igs, state, &xdev_gstate_procs);
-#endif
-    xdev_set_dest_inline(xdev, (Pixmap)0);
+
+/*  
+ * These matrix inits used to be to a private DGS data structure that was
+ * initialized at this point. We may need to ifdef DISPLAYGHOSTSCRIPT for
+ * proper initialization or maybe move to another part of the function ??? JC
+ */
+    xdev->initial_matrix.xx = 1.0;
+    xdev->initial_matrix.xy = 0.0;
+    xdev->initial_matrix.yx = 0.0;
+    xdev->initial_matrix.yy = 1.0;
+    xdev->initial_matrix.tx = 0.0;
+    xdev->initial_matrix.ty = 0.0;
+
+    xdev->dest = 0;
     if ((window_id = getenv("GHOSTVIEW"))) {
 	Window win;
 	Pixmap dest;
@@ -245,28 +232,28 @@ gdev_x_open(register gx_device_X * xdev)
 	    eprintf("Cannot get Window ID from ghostview.\n");
 	    return_error(gs_error_ioerror);
 	}
-	xdev_set_win_inline(xdev, win);
-	xdev_set_dest_inline(xdev, dest);
+	xdev->win = win;
+	xdev->dest = dest;
 	xdev->owner = gs_x_ghostview;
     }
     if (xdev->owner == gs_x_dpsagent) {
 	XWindowAttributes attrib;
 	/* recover the Window */
-	xdev_set_win_inline(xdev, xdev->pwin);
+	xdev->win = xdev->pwin;
 	xdev->pwin = (Window)None;
-	if (XGetWindowAttributes(xdev->dpy, xdev_win_inline(xdev), &attrib)) {
+	if (XGetWindowAttributes(xdev->dpy, xdev->win, &attrib)) {
 	    xdev->scr = attrib.screen;
 	    xvinfo.visual = attrib.visual;
-	    xdev_set_cmap_inline(xdev, attrib.colormap);
+	    xdev->cmap = attrib.colormap;
 	}
     } else if (xdev->pwin != (Window) None) {	/* pick up the destination window parameters if specified */
 	XWindowAttributes attrib;
 
-	xdev_set_win_inline(xdev, xdev->pwin);
-	if (XGetWindowAttributes(xdev->dpy, xdev_win_inline(xdev), &attrib)) {
+	xdev->win = xdev->pwin;
+	if (XGetWindowAttributes(xdev->dpy, xdev->win, &attrib)) {
 	    xdev->scr = attrib.screen;
 	    xvinfo.visual = attrib.visual;
-	    xdev_set_cmap_inline(xdev, attrib.colormap);
+	    xdev->cmap = attrib.colormap;
 	    xid_width = attrib.width;
 	    xid_height = attrib.height;
 	} else {
@@ -282,23 +269,22 @@ gdev_x_open(register gx_device_X * xdev)
 	char *buf;
 	Atom ghostview_atom = XInternAtom(xdev->dpy, "GHOSTVIEW", False);
 
-	if (XGetWindowAttributes(xdev->dpy, xdev_win_inline(xdev), &attrib)) {
+	if (XGetWindowAttributes(xdev->dpy, xdev->win, &attrib)) {
 	    xdev->scr = attrib.screen;
 	    xvinfo.visual = attrib.visual;
-	    xdev_set_cmap_inline(xdev, attrib.colormap);
+	    xdev->cmap = attrib.colormap;
 	    xdev->width = attrib.width;
 	    xdev->height = attrib.height;
 	}
 	/* Delete property if explicit dest is given */
-	if (XGetWindowProperty(xdev->dpy, xdev_win_inline(xdev), ghostview_atom, 0,
-			       256, (xdev_dest_inline(xdev) != 0), XA_STRING,
+	if (XGetWindowProperty(xdev->dpy, xdev->win, ghostview_atom, 0,
+			       256, (xdev->dest != 0), XA_STRING,
 			       &type, &format, &nitems, &bytes_after,
 			       (unsigned char **)&buf) == 0 &&
 	    type == XA_STRING) {
 	    int llx, lly, urx, ury;
 	    int left_margin = 0, bottom_margin = 0;
 	    int right_margin = 0, top_margin = 0;
-	    gs_matrix *initial_matrix;
 
 	    /* We declare page_orientation as an int so that we can */
 	    /* use an int * to reference it for sscanf; compilers */
@@ -306,11 +292,10 @@ gdev_x_open(register gx_device_X * xdev)
 	    /* it were declared as an orientation. */
 	    int /*orientation */ page_orientation;
 	    float xppp, yppp;	/* pixels per point */
-	    Pixmap bpixmap;
 
 	    nitems = sscanf(buf,
 			    "%ld %d %d %d %d %d %f %f %d %d %d %d",
-			    &bpixmap, &page_orientation,
+			    &(xdev->bpixmap), &page_orientation,
 			    &llx, &lly, &urx, &ury,
 			    &(xdev->x_pixels_per_inch),
 			    &(xdev->y_pixels_per_inch),
@@ -320,18 +305,17 @@ gdev_x_open(register gx_device_X * xdev)
 		eprintf("Cannot get ghostview property.\n");
 		return_error(gs_error_ioerror);
 	    }
-	    if (xdev_dest_inline(xdev) && bpixmap) {
+	    if (xdev->dest && xdev->bpixmap) {
 		eprintf("Both destination and backing pixmap specified.\n");
 		return_error(gs_error_rangecheck);
 	    }
-	    xdev_set_bpixmap_inline(xdev, bpixmap);
-	    if (xdev_dest_inline(xdev)) {
+	    if (xdev->dest) {
 		Window root;
 		int x, y;
 		unsigned int width, height;
 		unsigned int border_width, depth;
 
-		if (XGetGeometry(xdev->dpy, xdev_dest_inline(xdev), &root, &x, &y,
+		if (XGetGeometry(xdev->dpy, xdev->dest, &root, &x, &y,
 				 &width, &height, &border_width, &depth)) {
 		    xdev->width = width;
 		    xdev->height = height;
@@ -339,39 +323,38 @@ gdev_x_open(register gx_device_X * xdev)
 	    }
 	    xppp = xdev->x_pixels_per_inch / 72.0;
 	    yppp = xdev->y_pixels_per_inch / 72.0;
-	    initial_matrix = xdev_initial_matrix_inline(xdev);
 	    switch (page_orientation) {
 		case Portrait:
-		    initial_matrix->xx = xppp;
-		    initial_matrix->xy = 0.0;
-		    initial_matrix->yx = 0.0;
-		    initial_matrix->yy = -yppp;
-		    initial_matrix->tx = -llx * xppp;
-		    initial_matrix->ty = ury * yppp;
+		    xdev->initial_matrix.xx = xppp;
+		    xdev->initial_matrix.xy = 0.0;
+		    xdev->initial_matrix.yx = 0.0;
+		    xdev->initial_matrix.yy = -yppp;
+		    xdev->initial_matrix.tx = -llx * xppp;
+		    xdev->initial_matrix.ty = ury * yppp;
 		    break;
 		case Landscape:
-		    initial_matrix->xx = 0.0;
-		    initial_matrix->xy = yppp;
-		    initial_matrix->yx = xppp;
-		    initial_matrix->yy = 0.0;
-		    initial_matrix->tx = -lly * xppp;
-		    initial_matrix->ty = -llx * yppp;
+		    xdev->initial_matrix.xx = 0.0;
+		    xdev->initial_matrix.xy = yppp;
+		    xdev->initial_matrix.yx = xppp;
+		    xdev->initial_matrix.yy = 0.0;
+		    xdev->initial_matrix.tx = -lly * xppp;
+		    xdev->initial_matrix.ty = -llx * yppp;
 		    break;
 		case Upsidedown:
-		    initial_matrix->xx = -xppp;
-		    initial_matrix->xy = 0.0;
-		    initial_matrix->yx = 0.0;
-		    initial_matrix->yy = yppp;
-		    initial_matrix->tx = urx * xppp;
-		    initial_matrix->ty = -lly * yppp;
+		    xdev->initial_matrix.xx = -xppp;
+		    xdev->initial_matrix.xy = 0.0;
+		    xdev->initial_matrix.yx = 0.0;
+		    xdev->initial_matrix.yy = yppp;
+		    xdev->initial_matrix.tx = urx * xppp;
+		    xdev->initial_matrix.ty = -lly * yppp;
 		    break;
 		case Seascape:
-		    initial_matrix->xx = 0.0;
-		    initial_matrix->xy = -yppp;
-		    initial_matrix->yx = -xppp;
-		    initial_matrix->yy = 0.0;
-		    initial_matrix->tx = ury * xppp;
-		    initial_matrix->ty = urx * yppp;
+		    xdev->initial_matrix.xx = 0.0;
+		    xdev->initial_matrix.xy = -yppp;
+		    xdev->initial_matrix.yx = -xppp;
+		    xdev->initial_matrix.yy = 0.0;
+		    xdev->initial_matrix.tx = ury * xppp;
+		    xdev->initial_matrix.ty = urx * yppp;
 		    break;
 	    }
 
@@ -395,7 +378,7 @@ gdev_x_open(register gx_device_X * xdev)
 
 	xdev->scr = scr;
 	xvinfo.visual = DefaultVisualOfScreen(scr);
-	xdev_set_cmap_inline(xdev, DefaultColormapOfScreen(scr));
+	xdev->cmap = DefaultColormapOfScreen(scr);
     }
 
     xvinfo.visualid = XVisualIDFromVisual(xvinfo.visual);
@@ -422,11 +405,11 @@ gdev_x_open(register gx_device_X * xdev)
 
     /* Reserve foreground and background colors under the regular connection. */
     xc.pixel = xdev->foreground;
-    XQueryColor(xdev->dpy, xdev_cmap_inline(xdev), &xc);
-    XAllocColor(xdev->dpy, xdev_cmap_inline(xdev), &xc);
+    XQueryColor(xdev->dpy, xdev->cmap, &xc);
+    XAllocColor(xdev->dpy, xdev->cmap, &xc);
     xc.pixel = xdev->background;
-    XQueryColor(xdev->dpy, xdev_cmap_inline(xdev), &xc);
-    XAllocColor(xdev->dpy, xdev_cmap_inline(xdev), &xc);
+    XQueryColor(xdev->dpy, xdev->cmap, &xc);
+    XAllocColor(xdev->dpy, xdev->cmap, &xc);
 
     code = gdev_x_setup_colors(xdev);
     if (code < 0) {
@@ -519,10 +502,9 @@ gdev_x_open(register gx_device_X * xdev)
 	    if (bitmask & (XValue | YValue))
 		sizehints.flags |= USPosition;
 	}
-	gx_default_get_initial_matrix(dev, xdev_initial_matrix_inline(xdev));
+	gx_default_get_initial_matrix(dev, &(xdev->initial_matrix));
 
 	if (xdev->pwin != (Window) None && xid_width != 0 && xid_height != 0) {
-	    gs_matrix *initial_matrix = xdev_initial_matrix_inline(xdev);
 #if 0				/*************** */
 
 	    /*
@@ -537,23 +519,23 @@ gdev_x_open(register gx_device_X * xdev)
 	     */
 
 	    /* Scale to fit in the window. */
-	    initial_matrix.xx
-		= initial_matrix.xx *
+	    xdev->initial_matrix.xx
+		= xdev->initial_matrix.xx *
 		(float)xid_width / (float)xdev->width;
-	    initial_matrix.yy
-		= initial_matrix.yy *
+	    xdev->initial_matrix.yy
+		= xdev->initial_matrix.yy *
 		(float)xid_height / (float)xdev->height;
 
 #endif /*************** */
 	    xdev->width = xid_width;
 	    xdev->height = xid_height;
-	    initial_matrix->ty = xdev->height;
+	    xdev->initial_matrix.ty = xdev->height;
 	} else {		/* !xdev->pwin */
 	    xswa.event_mask = ExposureMask;
 	    xswa.background_pixel = xdev->background;
 	    xswa.border_pixel = xdev->borderColor;
-	    xswa.colormap = xdev_cmap_inline(xdev);
-	    xdev_set_win_inline(xdev, XCreateWindow(xdev->dpy, RootWindowOfScreen(xdev->scr),
+	    xswa.colormap = xdev->cmap;
+	    xdev->win = XCreateWindow(xdev->dpy, RootWindowOfScreen(xdev->scr),
 				      sizehints.x, sizehints.y,		/* upper left */
 				      xdev->width, xdev->height,
 				      xdev->borderWidth,
@@ -562,12 +544,12 @@ gdev_x_open(register gx_device_X * xdev)
 				      xdev->vinfo->visual,	/* visual */
 				      CWEventMask | CWBackPixel |
 				      CWBorderPixel | CWColormap,
-				      &xswa));
-	    XStoreName(xdev->dpy, xdev_win_inline(xdev), "ghostscript");
-	    XSetWMNormalHints(xdev->dpy, xdev_win_inline(xdev), &sizehints);
+				      &xswa);
+	    XStoreName(xdev->dpy, xdev->win, "ghostscript");
+	    XSetWMNormalHints(xdev->dpy, xdev->win, &sizehints);
 	    wm_hints.flags = InputHint;
 	    wm_hints.input = False;
-	    XSetWMHints(xdev->dpy, xdev_win_inline(xdev), &wm_hints); /* avoid input focus */
+	    XSetWMHints(xdev->dpy, xdev->win, &wm_hints); /* avoid input focus */
 	}
     }
     if (!resources_loaded) {
@@ -594,15 +576,15 @@ gdev_x_open(register gx_device_X * xdev)
     xdev->fid = (Font) 0;
 
     /* Set up a graphics context */
-    xdev_set_gc_inline(xdev, XCreateGC(xdev->dpy, xdev_win_inline(xdev), 0, (XGCValues *) NULL));
-    XSetFunction(xdev->dpy, xdev_gc_inline(xdev), GXcopy);
-    XSetLineAttributes(xdev->dpy, xdev_gc_inline(xdev), 0,
+    xdev->gc = XCreateGC(xdev->dpy, xdev->win, 0, (XGCValues *) NULL);
+    XSetFunction(xdev->dpy, xdev->gc, GXcopy);
+    XSetLineAttributes(xdev->dpy, xdev->gc, 0,
 		       LineSolid, CapButt, JoinMiter);
 
     gdev_x_clear_window(xdev);
 
     if (xdev->owner == gs_x_gs) {	/* Make the window appear. */
-	XMapWindow(xdev->dpy, xdev_win_inline(xdev));
+	XMapWindow(xdev->dpy, xdev->win);
 
 	/* Before anything else, do a flush and wait for */
 	/* an exposure event. */
@@ -613,8 +595,8 @@ gdev_x_open(register gx_device_X * xdev)
 	/* Now turn off graphics exposure events so they don't queue up */
 	/* indefinitely.  Also, since we can't do anything about real */
 	/* Expose events, mask them out. */
-	XSetGraphicsExposures(xdev->dpy, xdev_gc_inline(xdev), False);
-	XSelectInput(xdev->dpy, xdev_win_inline(xdev), NoEventMask);
+	XSetGraphicsExposures(xdev->dpy, xdev->gc, False);
+	XSelectInput(xdev->dpy, xdev->win, NoEventMask);
     } else {
 	/* Create an unmapped window, that the window manager will ignore.
 	 * This invisible window will be used to receive "next page"
@@ -631,7 +613,7 @@ gdev_x_open(register gx_device_X * xdev)
 	xdev->DONE = XInternAtom(xdev->dpy, "DONE", False);
     }
 
-    xdev->ht.no_pixmap = XCreatePixmap(xdev->dpy, xdev_win_inline(xdev), 1, 1,
+    xdev->ht.no_pixmap = XCreatePixmap(xdev->dpy, xdev->win, 1, 1,
 				       xdev->vinfo->depth);
 
     return 0;
@@ -645,49 +627,49 @@ gdev_x_clear_window(gx_device_X * xdev)
 	if (xdev->useBackingPixmap) {
 	    oldhandler = XSetErrorHandler(x_catch_alloc);
 	    alloc_error = False;
-	    xdev_set_bpixmap_inline(xdev,
-		XCreatePixmap(xdev->dpy, xdev_win_inline(xdev),
+	    xdev->bpixmap =
+		XCreatePixmap(xdev->dpy, xdev->win,
 			      xdev->width, xdev->height,
-			      xdev->vinfo->depth));
+			      xdev->vinfo->depth);
 	    XSync(xdev->dpy, False);	/* Force the error */
 	    if (alloc_error) {
 		xdev->useBackingPixmap = False;
 #ifdef DEBUG
 		eprintf("Warning: Failed to allocated backing pixmap.\n");
 #endif
-		if (xdev_bpixmap_inline(xdev)) {
-		    XFreePixmap(xdev->dpy, xdev_bpixmap_inline(xdev));
-		    xdev_set_bpixmap_inline(xdev, None);
+		if (xdev->bpixmap) {
+		    XFreePixmap(xdev->dpy, xdev->bpixmap);
+		    xdev->bpixmap = None;
 		    XSync(xdev->dpy, False);	/* Force the error */
 		}
 	    }
 	    oldhandler = XSetErrorHandler(oldhandler);
 	} else
-	    xdev_set_bpixmap_inline(xdev, (Pixmap) 0);
+	    xdev->bpixmap = (Pixmap) 0;
     }
     /* Clear the destination pixmap to avoid initializing with garbage. */
-    if (xdev_dest_inline(xdev) != (Pixmap) 0) {
-	XSetForeground(xdev->dpy, xdev_gc_inline(xdev), xdev->background);
-	XFillRectangle(xdev->dpy, xdev_dest_inline(xdev), xdev_gc_inline(xdev),
+    if (xdev->dest != (Pixmap) 0) {
+	XSetForeground(xdev->dpy, xdev->gc, xdev->background);
+	XFillRectangle(xdev->dpy, xdev->dest, xdev->gc,
 		       0, 0, xdev->width, xdev->height);
     } else {
-	xdev_set_dest_inline(xdev, (xdev_bpixmap_inline(xdev) != (Pixmap) 0 ?
-			     xdev_bpixmap_inline(xdev) : (Pixmap) xdev_win_inline(xdev)));
+	xdev->dest = (xdev->bpixmap != (Pixmap) 0 ?
+			     xdev->bpixmap : (Pixmap) xdev->win);
     }
 
     /* Clear the background pixmap to avoid initializing with garbage. */
-    if (xdev_bpixmap_inline(xdev) != (Pixmap) 0) {
+    if (xdev->bpixmap != (Pixmap) 0) {
 	if (xdev->owner == gs_x_gs)
-	    XSetWindowBackgroundPixmap(xdev->dpy, xdev_win_inline(xdev), xdev_bpixmap_inline(xdev));
-	XSetForeground(xdev->dpy, xdev_gc_inline(xdev), xdev->background);
-	XFillRectangle(xdev->dpy, xdev_bpixmap_inline(xdev), xdev_gc_inline(xdev),
+	    XSetWindowBackgroundPixmap(xdev->dpy, xdev->win, xdev->bpixmap);
+	XSetForeground(xdev->dpy, xdev->gc, xdev->background);
+	XFillRectangle(xdev->dpy, xdev->bpixmap, xdev->gc,
 		       0, 0, xdev->width, xdev->height);
     }
     /* Initialize foreground and background colors */
     xdev->back_color = xdev->background;
-    XSetBackground(xdev->dpy, xdev_gc_inline(xdev), xdev->background);
+    XSetBackground(xdev->dpy, xdev->gc, xdev->background);
     xdev->fore_color = xdev->background;
-    XSetForeground(xdev->dpy, xdev_gc_inline(xdev), xdev->background);
+    XSetForeground(xdev->dpy, xdev->gc, xdev->background);
     xdev->colors_or = xdev->colors_and = xdev->background;
 }
 
@@ -704,11 +686,14 @@ x_get_std_cmap(gx_device_X * xdev, Atom prop)
 
     if (XGetRGBColormaps(xdev->dpy, RootWindowOfScreen(xdev->scr),
 			 &scmap, &nitems, prop))
+
+    /* Why is this different for DISPLAYGHOSTSCRIPT? */
 	for (i = 0, sp = scmap; i < nitems; i++, sp++) {
-	    xdev_set_cmap_inline(xdev, sp->colormap);
-	    if (xdev_cmap_inline(xdev))
+	    xdev->cmap = sp->colormap;
+	    if (xdev->cmap)
 		return sp;
 	}
+
     return NULL;
 }
 #endif
@@ -739,7 +724,7 @@ private void
 free_ramp(gx_device_X * xdev, int num_used, int size)
 {
     if (num_used - 1 > 0) {
-	XFreeColors(xdev->dpy, xdev_cmap_inline(xdev),
+	XFreeColors(xdev->dpy, xdev->cmap,
 		    xdev->dither_colors + 1,
 		    num_used - 1, 0);
     }
@@ -783,7 +768,7 @@ setup_cube(gx_device_X * xdev, int ramp_size, bool colors)
 	xc.red = (X_max_color_value * r / max_rgb) & xdev->color_mask;
 	xc.green = (X_max_color_value * g / max_rgb) & xdev->color_mask;
 	xc.blue = (X_max_color_value * b / max_rgb) & xdev->color_mask;
-	if (XAllocColor(xdev->dpy, xdev_cmap_inline(xdev), &xc)) {
+	if (XAllocColor(xdev->dpy, xdev->cmap, &xc)) {
 	    xdev->dither_colors[index] = xc.pixel;
 	} else {
 	    free_ramp(xdev, index, num_entries);
@@ -812,8 +797,8 @@ gdev_x_setup_colors(gx_device_X * xdev)
 	char *buf;
 
 	/* Delete property if explicit dest is given */
-	if (XGetWindowProperty(xdev->dpy, xdev_win_inline(xdev), gv_colors, 0,
-			       256, (xdev_dest_inline(xdev) != 0), XA_STRING,
+	if (XGetWindowProperty(xdev->dpy, xdev->win, gv_colors, 0,
+			       256, (xdev->dest != 0), XA_STRING,
 			       &type, &format, &nitems, &bytes_after,
 			       (unsigned char **)&buf) == 0 &&
 	    type == XA_STRING) {
