@@ -21,7 +21,7 @@
   does not depend on any other GNU software.
 */
 
-/*$Id: gsstate.c,v 1.8 1999/05/30 18:58:32 masata-y Exp $ */
+/*$Id: gsstate.c,v 1.9 2000/04/04 17:02:32 cigas Exp $ */
 /* Miscellaneous graphics state operators for Ghostscript library */
 #include "gx.h"
 #include "memory_.h"
@@ -103,7 +103,7 @@ private int gstate_copy(P4(gs_state *, const gs_state *,
  *
  * (4) Objects that are referenced directly by exactly one gstate and that
  *      are not referenced (except transiently) from any other object.
- *      These fall into three groups:
+ *      These fall into two groups:
  *
  *   (4b) Objects allocated individually, for the given reason:
  *              line_params.dash.pattern (variable-length),
@@ -115,10 +115,6 @@ private int gstate_copy(P4(gs_state *, const gs_state *,
  *      the refs associated with the gstate, such as the screen procedures.
  *      Client-supplied procedures manage client data.
  *
- *   (4c) The "driver data" for a gstate.  For the X11 driver, this contains
- *      window, GC, etc. as they are gstate-specific.
- *      Driver-supplied procedures manage client data.
- *
  * (5) Objects referenced indirectly from gstate objects of category (4),
  *      including objects that may also be referenced directly by the gstate.
  *      The individual routines that manipulate these are responsible
@@ -153,9 +149,9 @@ private int gstate_copy(P4(gs_state *, const gs_state *,
 #define gs_state_do_ptrs(m)\
   m(0,saved) m(1,path) m(2,clip_path) m(3,view_clip) m(4,effective_clip_path)\
   m(5,color_space) m(6,ccolor) m(7,dev_color)\
-  m(8,font) m(9,root_font) m(10,show_gstate)\
-  m(11,client_data) /* m(12,driver_data) */ /*m(---,device)*/
-#define gs_state_num_ptrs (13-1)
+  m(8,font) m(9,root_font) m(10,show_gstate) /*m(---,device)*/\
+  m(11,client_data)
+#define gs_state_num_ptrs 12
 
 /*
  * Define these elements of the graphics state that are allocated
@@ -236,18 +232,6 @@ gstate_copy_client_data(gs_state * pgs, void *dto, void *dfrom,
 	    (*pgs->client_procs.copy) (dto, dfrom));
 }
 
-/* Copy driver data, using the copy_for procedure if available, */
-/* the copy procedure otherwise. */
-private int
-gstate_copy_driver_data(gs_state * pgs, void *dto, void *dfrom,
-			gs_state_copy_reason_t reason)
-{
-    return (pgs->driver_procs.copy_for != 0 ?
-	    (*pgs->driver_procs.copy_for) (dto, dfrom, reason) :
-	    (*pgs->driver_procs.copy) (dto, dfrom));
-}
-
-
 /* ------ Operations on the entire graphics state ------ */
 
 /* Initialize an imager state, other than the parts covered by */
@@ -389,7 +373,6 @@ gs_state_alloc(gs_memory_t * mem)
 				      gx_ht_cache_default_bits());
     gs_imager_state_initialize((gs_imager_state *) pgs, mem);
     pgs->client_data = 0;
-    pgs->driver_data = 0;
 
     /* Initialize other things not covered by initgraphics */
 
@@ -421,7 +404,6 @@ gs_state_alloc(gs_memory_t * mem)
     pgs->show_gstate = 0;
     pgs->level = 0;
     pgs->client_data = 0;
-    pgs->driver_data = 0;
     if (gs_initgraphics(pgs) < 0) {
 	/* Something went very wrong */
 	return 0;
@@ -450,23 +432,6 @@ gs_state_client_data(const gs_state * pgs)
     return pgs->client_data;
 }
 
-/* Set the driver data in a graphics state. */
-/* This should only be done to a newly created state. */
-void
-gs_state_set_driver(gs_state * pgs, void *pdata,
-		    const gs_state_driverdata_procs * pprocs)
-{
-    pgs->driver_data = pdata;
-    pgs->driver_procs = *pprocs;
-}
-
-/* Get the driver data from a graphics state. */
-void *
-gs_state_driver_data(const gs_state * pgs)
-{
-    return pgs->driver_data;
-}
-
 /* Free a graphics state. */
 int
 gs_state_free(gs_state * pgs)
@@ -533,37 +498,26 @@ int
 gs_grestore(gs_state * pgs)
 {
     gs_state *saved = pgs->saved;
-    void *pdata, *pdata_driver;
-    void *sdata, *sdata_driver;
+    void *pdata = pgs->client_data;
+    void *sdata;
 
-    if (pgs->driver_data)
-	(*pgs->driver_procs.willchange)(pgs, pgs->driver_data);
-    pdata = pgs->client_data;
-    pdata_driver = pgs->driver_data;
     if_debug2('g', "[g]grestore 0x%lx, level was %d\n",
 	      (ulong) saved, pgs->level);
     if (!saved)			/* shouldn't happen */
 	return gs_gsave(pgs);
     sdata = saved->client_data;
-    sdata_driver = saved->driver_data;    
     if (saved->pattern_cache == 0)
 	saved->pattern_cache = pgs->pattern_cache;
     /* Swap back the client data pointers. */
     pgs->client_data = sdata;
     saved->client_data = pdata;
-    pgs->driver_data = sdata_driver;
-    saved->driver_data = pdata_driver;
     if (pdata != 0 && sdata != 0)
 	gstate_copy_client_data(pgs, pdata, sdata, copy_for_grestore);
-    if (pdata_driver != 0 && sdata_driver != 0)
-      gstate_copy_driver_data(pgs, pdata_driver, sdata_driver, copy_for_grestore);
     gstate_free_contents(pgs);
     *pgs = *saved;
     if (pgs->show_gstate == saved)
 	pgs->show_gstate = pgs;
     gs_free_object(pgs->memory, saved, "gs_grestore");
-    if (pgs->driver_data && pgs->driver_procs.didchange)
-	(*pgs->driver_procs.didchange)(pgs, pgs->driver_data);
     if (pgs->saved)
 	return 0;
     return gs_gsave(pgs);
@@ -680,8 +634,6 @@ gs_setgstate(gs_state * pgs, const gs_state * pfrom)
     gx_clip_path *view_clip = pgs->view_clip;
     int code;
 
-    if (pgs->driver_data)
-      (*pgs->driver_procs.willchange)(pgs, pgs->driver_data);
     pgs->view_clip = 0;		/* prevent refcount decrementing */
     code = gstate_copy(pgs, pfrom, copy_for_setgstate, "gs_setgstate");
     if (code < 0)
@@ -690,8 +642,6 @@ gs_setgstate(gs_state * pgs, const gs_state * pfrom)
     pgs->view_clip = view_clip;
     pgs->show_gstate =
 	(pgs->show_gstate == pfrom ? pgs : saved_show);
-    if (pgs->driver_data && pgs->driver_procs.didchange)
-      (*pgs->driver_procs.didchange)(pgs, pgs->driver_data);    
     return 0;
 }
 
@@ -914,14 +864,6 @@ gstate_clone(gs_state * pfrom, gs_memory_t * mem, client_name_t cname,
 	    )
 	    goto fail;
     }
-    if (pgs->driver_data != 0) {
-	void *pdata_driver = pgs->driver_data = (*pgs->driver_procs.alloc) (mem);
-        if (pdata_driver == 0 ||
-	    gstate_copy_driver_data(pgs, pdata_driver, pfrom->driver_data, reason) < 0
-	    )
-	    goto fail;
-    }
-
     rc_increment(pgs->set_transfer.colored.gray);
     rc_increment(pgs->set_transfer.colored.red);
     rc_increment(pgs->set_transfer.colored.green);
@@ -992,8 +934,6 @@ gstate_free_contents(gs_state * pgs)
     cs_adjust_counts(pgs, -1);
     if (pgs->client_data != 0)
 	(*pgs->client_procs.free) (pgs->client_data, mem);
-    if (pgs->driver_data != 0)
-	(*pgs->driver_procs.free) (pgs->driver_data, mem);    
     gs_free_object(mem, pgs->line_params.dash.pattern, cname);
     gstate_free_parts(pgs, mem, cname);
 #undef RCDECR
@@ -1060,16 +1000,12 @@ gstate_copy(gs_state * pto, const gs_state * pfrom,
 	struct gx_pattern_cache_s *pcache = pto->pattern_cache;
 /*! BUG: what if pto->client_data isn't alloced yet!? */
 	void *pdata = pto->client_data;
-	void *pdata_driver = pto->driver_data;
 	gs_memory_t *mem = pto->memory;
 	gs_state *saved = pto->saved;
 	float *pattern = pto->line_params.dash.pattern;
 
-	if ( pfrom->driver_data != 0 )
-	    (*pfrom->driver_procs.willchange)(pfrom, pfrom->driver_data);
 	*pto = *pfrom;
 	pto->client_data = pdata;
-	pto->driver_data = pdata_driver;	
 	pto->memory = mem;
 	pto->saved = saved;
 	pto->line_params.dash.pattern = pattern;
@@ -1080,23 +1016,6 @@ gstate_copy(gs_state * pto, const gs_state * pfrom,
 	    gstate_copy_client_data((gs_state *) pfrom, pdata,
 				    pfrom->client_data, reason);
 	}
-	if (pdata_driver != 0) {
-	  /* Alloc a space for copying */
-	  if (pdata_driver == NULL)
-	    {
-	      pdata_driver = pto->driver_procs.alloc(pto->memory);
-	      if (!pdata_driver)
-		{
-		  eprintf("gs: cannot allocate state for X device.\n");
-		  return_error(gs_error_VMerror);
-		}
-	    }
-	  /* We need to break 'const' here. */
-	  gstate_copy_driver_data((gs_state *) pfrom, pdata_driver,
-				  pfrom->driver_data, reason);
-	  if (pfrom->driver_procs.didchange != 0)
-	    (*pfrom->driver_procs.didchange)(pto, pdata_driver);
-	}
     }
     GSTATE_ASSIGN_PARTS(pto, &parts);
 #undef RCCOPY
