@@ -21,7 +21,7 @@
   does not depend on any other GNU software.
 */
 
-/*$Id: interp.c,v 1.8 2000/01/05 17:12:45 masata-y Exp $ */
+/*$Id: interp.c,v 1.9 2000/02/21 03:46:19 cigas Exp $ */
 /* Ghostscript language interpreter */
 #include "memory_.h"
 #include "string_.h"
@@ -691,10 +691,10 @@ interp(ref * pref /* object to interpret */ , ref * perror_object)
 #define return_with_stackoverflow_iref()\
   { o_stack.requested = 1; return_with_error_iref(e_stackoverflow); }
 
-    extern void ticks_left_set(int value);
-    extern int ticks_left_decrement(void);
-    extern int ticks_left_value(void);
-    extern int * ticks_left_refer(void);
+    void ticks_left_set(int value);
+    int ticks_left_decrement(void);
+    int ticks_left_value(void);
+    int * ticks_left_refer(void);
     const int gc_ticks_val = -20;
     int slice_counter = 0;
     const int slice_counter_max = 10000;
@@ -1565,3 +1565,37 @@ const op_def interp_op_defs[] =
     {"0%oparray_pop", oparray_pop},
     op_def_end(0)
 };
+
+/* --- Context switching via time slice control --- */
+private int ticks_left;
+
+void
+ticks_left_set(int value)
+{
+  ticks_left = value;
+}
+
+void
+ticks_left_zero(void)
+{
+  ticks_left_set(0);
+}
+
+int
+ticks_left_decrement(void)
+{
+  return --ticks_left;
+}
+
+
+int
+ticks_left_value(void)
+{
+  return ticks_left;
+}
+
+int *
+ticks_left_refer(void)
+{
+  return &ticks_left;
+}
